@@ -81,6 +81,7 @@ void dbd_register_driver(const dbi_info_t **_driver_info, const char ***_custom_
 
 int dbd_initialize(dbi_driver_t *driver) 
 {
+        _dbd_register_driver_cap(driver, "safe_dlclose", 1);
         return 0;
 }
 
@@ -270,7 +271,7 @@ dbi_result_t *dbd_query(dbi_conn_t *conn, const char *statement)
 const char *dbd_select_db(dbi_conn_t *conn, const char *db) 
 {
 	if (msqlSelectDB((int)conn->connection, (char *)db) < 0) {
-		_error_handler(conn, DBI_ERROR_DBD);
+		_dbd_internal_error_handler(conn, NULL, DBI_ERROR_DBD);
 		return "";
 	}
 	return db;
@@ -289,13 +290,15 @@ int dbd_geterror(dbi_conn_t *conn, int *errno, char **errstr)
 
 unsigned long long dbd_get_seq_last(dbi_conn_t *conn, const char *sequence) 
 {  
-	return 0;
+  _dbd_internal_error_handler(conn, NULL, DBI_ERROR_UNSUPPORTED);
+  return 0;
 }
 
 
 unsigned long long dbd_get_seq_next(dbi_conn_t *conn, const char *sequence) 
 {	
-	return 0;
+  _dbd_internal_error_handler(conn, NULL, DBI_ERROR_UNSUPPORTED);
+  return 0;
 }
 
 
