@@ -21,7 +21,7 @@
  * Copyright (C) 2001-2002, Mark Tobenkin <mark@brentwoodradio.com>
  * http://libdbi.sourceforge.net
  * 
- * $Id: dbd_mysql.c,v 1.90 2006/11/29 21:11:41 mhoenicka Exp $
+ * $Id: dbd_mysql.c,v 1.95 2008/01/15 00:27:29 mhoenicka Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,10 @@ long long atoll(const char *str);
 long long strtoll(const char *nptr, char **endptr, int base);
 #endif
 
+#ifdef __MINGW32__
+#include <winsock.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -122,13 +126,14 @@ int dbd_initialize(dbi_driver_t *driver) {
 	 * this is called right after dbd_register_driver().
 	 * return -1 on error, 0 on success. if -1 is returned, the driver will not
 	 * be added to the list of available drivers. */
-	
+        _dbd_register_driver_cap(driver, "safe_dlclose", 1);
 	return 0;
 }
 
 int dbd_connect(dbi_conn_t *conn) {
 	MYSQL *mycon;
 	char* sql_cmd;
+	unsigned long client_flags = 0;
 
 	const char *host = dbi_conn_get_option(conn, "host");
 	const char *username = dbi_conn_get_option(conn, "username");
@@ -138,17 +143,28 @@ int dbd_connect(dbi_conn_t *conn) {
 	int port = dbi_conn_get_option_numeric(conn, "port");
 	/* mysql specific options */
 	const char *unix_socket = dbi_conn_get_option(conn, "mysql_unix_socket");
-	int compression = dbi_conn_get_option_numeric(conn, "mysql_compression");
-
-	int _compression = (compression > 0) ? CLIENT_COMPRESS : 0;
+	/* we honor the mysql_compression flag for the sake of
+	   backwards compatibility. Newer code should use the
+	   mysql_client_whatever flags instead */
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_compression") > 0) ? CLIENT_COMPRESS : 0;
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_client_compress") > 0) ? CLIENT_COMPRESS : 0;
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_client_found_rows") > 0) ? CLIENT_FOUND_ROWS : 0;
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_client_ignore_SPACE") > 0) ? CLIENT_IGNORE_SPACE : 0;
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_client_interactive") > 0) ? CLIENT_INTERACTIVE : 0;
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_client_local_files") > 0) ? CLIENT_LOCAL_FILES : 0;
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_client_multi_statements") > 0) ? CLIENT_MULTI_STATEMENTS : 0;
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_client_multi_results") > 0) ? CLIENT_MULTI_RESULTS : 0;
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_client_no_schema") > 0) ? CLIENT_NO_SCHEMA : 0;
+	client_flags |= (dbi_conn_get_option_numeric(conn, "mysql_client_odbc") > 0) ? CLIENT_ODBC : 0;
 	
 	mycon = mysql_init(NULL);
 	if (!mycon) {
-		return -1;
+		_dbd_internal_error_handler(conn, NULL, DBI_ERROR_NOMEM);
+		return -2;
 	}
-	else if (!mysql_real_connect(mycon, host, username, password, dbname, port, unix_socket, _compression)) {
+	else if (!mysql_real_connect(mycon, host, username, password, dbname, port, unix_socket, client_flags)) {
 		conn->connection = (void *)mycon; // still need this set so _error_handler can grab information
-		_error_handler(conn, DBI_ERROR_DBD);
+		_dbd_internal_error_handler(conn, NULL, DBI_ERROR_DBD);
 		mysql_close(mycon);
 		conn->connection = NULL; // myconn no longer valid
 		return -2;
@@ -157,14 +173,14 @@ int dbd_connect(dbi_conn_t *conn) {
 		conn->connection = (void *)mycon;
 		if (dbname) conn->current_db = strdup(dbname);
 	}
-	
+/* 	printf("dbname went to %s\n", dbname);	 */
 	if (encoding && *encoding) {
 	  /* set connection encoding */
 	  if (!strcmp(encoding, "auto")) {
 	    encoding = dbd_get_encoding(conn);
 	    if (encoding) {
-	      asprintf(&sql_cmd, "SET NAMES '%s'", encoding);
-/* 	      printf("SET NAMES '%s'", encoding); */
+	      asprintf(&sql_cmd, "SET NAMES '%s'", dbd_encoding_from_iana(encoding));
+/*  	      printf("SET NAMES '%s'\n", dbd_encoding_from_iana(encoding)); */
 	      dbd_query(conn, sql_cmd);
 	      free(sql_cmd);
 	    }
@@ -175,11 +191,11 @@ int dbd_connect(dbi_conn_t *conn) {
 	  }
 	  else {
 	    asprintf(&sql_cmd, "SET NAMES '%s'", dbd_encoding_from_iana(encoding));
-/*  	    printf("SET NAMES '%s'", dbd_encoding_from_iana(encoding)); */
+/*   	    printf("SET NAMES '%s'", dbd_encoding_from_iana(encoding)); */
 	    dbd_query(conn, sql_cmd);
 	    free(sql_cmd);
 	  }
-/* 	  printf("set encoding to %s<<\n", encoding); */
+/*  	  printf("set encoding to %s<<\n", dbd_encoding_from_iana(encoding)); */
 	}
 /* 	else { */
 /* 	  printf("leave encoding alone\n"); */
@@ -526,7 +542,7 @@ dbi_result_t *dbd_query_null(dbi_conn_t *conn, const unsigned char *statement, s
 
 const char *dbd_select_db(dbi_conn_t *conn, const char *db) {
 	if (mysql_select_db((MYSQL *)conn->connection, db)) {
-		_error_handler(conn, DBI_ERROR_DBD);
+		_dbd_internal_error_handler(conn, NULL, DBI_ERROR_DBD);
 		return "";
 	}
 
