@@ -1,28 +1,28 @@
 /*
  * libdbi-drivers - 3rd-party drivers for libdbi, a database independent
  * abstraction layer for C.
- * Copyright (C) 2002, Markus Hoenicka
+ * Copyright (C) 2002-2007, Markus Hoenicka
  * http://libdbi-drivers.sourceforge.net
  * 
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *  
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- * 
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * dbd_sqlite.c: SQLite database support (using libsqlite)
- * Copyright (C) 2002, Markus Hoenicka <mhoenicka@users.sourceforge.net>
+ * Copyright (C) 2002-2007, Markus Hoenicka <mhoenicka@users.sourceforge.net>
  * http://libdbi-drivers.sourceforge.net
  * 
- * $Id: dbd_sqlite.c,v 1.35 2006/10/17 19:18:47 mhoenicka Exp $
+ * $Id: dbd_sqlite.c,v 1.39 2008/01/15 14:07:24 mhoenicka Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -100,6 +100,7 @@ int dbd_initialize(dbi_driver_t *driver) {
    * return -1 on error, 0 on success. if -1 is returned, the driver will not
    * be added to the list of available drivers. */
 	
+  _dbd_register_driver_cap(driver, "safe_dlclose", 1);
   return 0;
 }
 
@@ -139,7 +140,7 @@ int _real_dbd_connect(dbi_conn_t *conn, const char* database) {
   }
 
   if (!dbname) {
-    _dbd_internal_error_handler(conn, "no database specified", 0);
+    _dbd_internal_error_handler(conn, "no database specified", DBI_ERROR_CLIENT);
     return -1;
   }
 
@@ -147,7 +148,7 @@ int _real_dbd_connect(dbi_conn_t *conn, const char* database) {
   dbdir = dbi_conn_get_option(conn, "sqlite_dbdir");
 	
   if (!dbdir) {
-    _dbd_internal_error_handler(conn, "no database directory specified", 0);
+    _dbd_internal_error_handler(conn, "no database directory specified", DBI_ERROR_CLIENT);
     return -1;
   }
 
@@ -156,7 +157,7 @@ int _real_dbd_connect(dbi_conn_t *conn, const char* database) {
   db_fullpath = malloc(strlen(dbname)+strlen(dbdir)+2); /* leave room
 							   for \0 and / */
   if (db_fullpath == NULL) {
-    _dbd_internal_error_handler(conn, "out of memory", 0);
+    _dbd_internal_error_handler(conn, NULL, DBI_ERROR_NOMEM);
     return -1;
   }
 
@@ -183,7 +184,7 @@ int _real_dbd_connect(dbi_conn_t *conn, const char* database) {
        access rights or an existing database is corrupted or created
        with an incompatible version */
     if (sq_errmsg) {
-      _dbd_internal_error_handler(conn, sq_errmsg, 0);
+      _dbd_internal_error_handler(conn, sq_errmsg, DBI_ERROR_CLIENT);
       free(sq_errmsg);
     }
     else {
@@ -324,7 +325,7 @@ dbi_result_t *dbd_list_dbs(dbi_conn_t *conn, const char *pattern) {
   dbd_query(conn, "CREATE TEMPORARY TABLE libdbi_databases (dbname VARCHAR(255))");
 
   if (sq_datadir && (dp = opendir(sq_datadir)) == NULL) {
-    _dbd_internal_error_handler(conn, "could not open data directory", 0);
+    _dbd_internal_error_handler(conn, "could not open data directory", DBI_ERROR_CLIENT);
     return NULL;
   }
   getcwd(old_cwd, _POSIX_PATH_MAX);
@@ -416,7 +417,7 @@ dbi_result_t *dbd_list_tables(dbi_conn_t *conn, const char *db, const char *patt
   dbi_conn_set_option(tempconn, "sqlite_dbdir", (char*)dbi_conn_get_option(conn, "sqlite_dbdir"));
 
   if (dbi_conn_connect(tempconn) < 0) {
-    _dbd_internal_error_handler(conn, "could not connect to database", 0);
+    _dbd_internal_error_handler(conn, NULL, DBI_ERROR_NOCONN);
     return NULL;
   }
   
@@ -932,6 +933,7 @@ unsigned long long dbd_get_seq_last(dbi_conn_t *conn, const char *sequence) {
 }
 
 unsigned long long dbd_get_seq_next(dbi_conn_t *conn, const char *sequence) {
+  _dbd_internal_error_handler(conn, NULL, DBI_ERROR_UNSUPPORTED);
   return 0;
 }
 
